@@ -1,3 +1,6 @@
+#ifndef QT_FEATURES_GlobalConfig_src_corelib_global_qconfig_p_h_H
+#define QT_FEATURES_GlobalConfig_src_corelib_global_qconfig_p_h_H
+
 #define QT_FEATURE_use_bfd_linker -1
 
 #define QT_FEATURE_use_gold_linker -1
@@ -80,13 +83,13 @@
 
 #define QT_FEATURE_mips_dspr2 -1
 
-#define QT_FEATURE_neon -1
+#define QT_FEATURE_neon 1
 
 #define QT_FEATURE_arm_crc32 -1
 
 #define QT_FEATURE_arm_crypto 1
 
-#define QT_FEATURE_arm_sve -1
+#define QT_FEATURE_arm_sve 1
 
 #define QT_FEATURE_localtime_r 1
 
@@ -142,5 +145,7 @@
 
 #define QT_FEATURE_libcpp_hardening -1
 
-#define QT_FEATURE_relro_now_linker -1
+#define QT_FEATURE_relro_now_linker 1
 
+
+#endif // QT_FEATURES_GlobalConfig_src_corelib_global_qconfig_p_h_H
