@@ -1,3 +1,6 @@
+#ifndef QT_FEATURES_Core_src_corelib_qtcore_config_p_h_H
+#define QT_FEATURES_Core_src_corelib_qtcore_config_p_h_H
+
 #define QT_FEATURE_clock_gettime 1
 
 #define QT_FEATURE_copy_file_range -1
@@ -48,7 +51,7 @@
 
 #define QT_FEATURE_poll_select -1
 
-#define QT_FEATURE_posix_sem 1
+#define QT_FEATURE_posix_sem -1
 
 #define QT_FEATURE_posix_shm -1
 
@@ -104,3 +107,5 @@
 
 #define QT_FEATURE_openssl_hash -1
 
+
+#endif // QT_FEATURES_Core_src_corelib_qtcore_config_p_h_H
