@@ -1,3 +1,6 @@
+#ifndef QT_FEATURES_Core_src_corelib_qtcore_config_h_H
+#define QT_FEATURES_Core_src_corelib_qtcore_config_h_H
+
 #define QT_FEATURE_clock_monotonic 1
 
 #define QT_FEATURE_cxx11_future 1
@@ -88,3 +91,5 @@
 
 #define QT_NO_INOTIFY 1
 #define QT_THREADSAFE_CLOEXEC 1
+
+#endif // QT_FEATURES_Core_src_corelib_qtcore_config_h_H
