@@ -1,6 +1,7 @@
 // Copyright (C) 2018 Intel Corporation.
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #ifndef QCBORCOMMON_P_H
 #define QCBORCOMMON_P_H
@@ -32,6 +33,7 @@ QT_WARNING_DISABLE_GCC("-Wunused-function")
 QT_WARNING_DISABLE_CLANG("-Wunused-function")
 QT_WARNING_DISABLE_CLANG("-Wundefined-internal")
 
+#define CBOR_NO_HALF_FLOAT_TYPE 1
 #define CBOR_NO_VALIDATION_API  1
 #define CBOR_NO_PRETTY_API      1
 #define CBOR_API static inline
