@@ -1,5 +1,10 @@
+#ifndef QT_FEATURES_Core_src_corelib_qtcore_config_p_h_H
+#define QT_FEATURES_Core_src_corelib_qtcore_config_p_h_H
+
 #define QT_FEATURE_clock_gettime 1
 
+#define QT_FEATURE_copy_file_range -1
+
 #define QT_FEATURE_doubleconversion 1
 
 #define QT_FEATURE_system_doubleconversion 1
@@ -18,8 +23,12 @@
 
 #define QT_FEATURE_icu 1
 
+#define QT_FEATURE_winsdkicu -1
+
 #define QT_FEATURE_inotify -1
 
+#define QT_FEATURE_jemalloc -1
+
 #define QT_FEATURE_journald -1
 
 #define QT_FEATURE_system_libb2 1
@@ -46,6 +55,12 @@
 
 #define QT_FEATURE_posix_shm 1
 
+#define QT_FEATURE_pthread_clockjoin -1
+
+#define QT_FEATURE_pthread_condattr_setclock 1
+
+#define QT_FEATURE_pthread_timedjoin -1
+
 #define QT_FEATURE_qqnx_pps -1
 
 #define QT_FEATURE_renameat2 -1
@@ -58,8 +73,16 @@
 
 #define QT_FEATURE_sysv_shm 1
 
+#define QT_FEATURE_dup3 1
+
+#define QT_FEATURE_accept4 1
+
+#define QT_FEATURE_vxpipedrv -1
+
 #define QT_FEATURE_backtrace 1
 
+#define QT_FEATURE_cxx20_format 1
+
 #define QT_FEATURE_cxx23_stacktrace -1
 
 #define QT_FEATURE_cpp_winrt -1
@@ -84,3 +107,5 @@
 
 #define QT_FEATURE_openssl_hash -1
 
+
+#endif // QT_FEATURES_Core_src_corelib_qtcore_config_p_h_H
