@@ -1,3 +1,6 @@
+#ifndef QT_FEATURES_GlobalConfig_src_corelib_global_qconfig_h_H
+#define QT_FEATURES_GlobalConfig_src_corelib_global_qconfig_h_H
+
 #define QT_FEATURE_version_tagging 1
 
 #define QT_FEATURE_shared 1
@@ -28,6 +31,8 @@
 
 #define QT_FEATURE_cxx2b -1
 
+#define QT_FEATURE_cxx2c -1
+
 #define QT_FEATURE_reduce_relocations -1
 
 #define QT_FEATURE_signaling_nan 1
@@ -36,6 +41,8 @@
 
 #define QT_FEATURE_wasm_exceptions -1
 
+#define QT_FEATURE_wasm_jspi -1
+
 #define QT_FEATURE_zstd 1
 
 #define QT_FEATURE_thread 1
@@ -52,18 +59,21 @@
 
 #define QT_FEATURE_opensslv30 -1
 
+#define QT_FEATURE_test_gui 1
 
-#define QT_VERSION_STR "6.8.3"
+
+#define QT_VERSION_STR "6.10.1"
 
 #define QT_VERSION_MAJOR 6
 
-#define QT_VERSION_MINOR 8
+#define QT_VERSION_MINOR 10
 
-#define QT_VERSION_PATCH 3
+#define QT_VERSION_PATCH 1
 
 #define QT_COPYRIGHT "Copyright (C) The Qt Company Ltd. and other contributors."
 #define QT_LARGEFILE_SUPPORT 64
 #define QT_VISIBILITY_AVAILABLE
-#define QT_USE_PROTECTED_VISIBILITY
-#define QT_COMPILER_SUPPORTS_NEON 1
 #define QT_LINKED_OPENSSL
+#define QT_GUI_TEST 1
+
+#endif // QT_FEATURES_GlobalConfig_src_corelib_global_qconfig_h_H
