@@ -19,7 +19,11 @@
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
+#include FT_MULTIPLE_MASTERS_H
 
+#if defined(FT_COLOR_H)
+#  include FT_COLOR_H
+#endif
 
 #ifndef Q_OS_WIN
 #include <unistd.h>
@@ -28,11 +32,17 @@
 #include <qmutex.h>
 
 #include <string.h>
+#include <qpainterpath.h>
 
 QT_BEGIN_NAMESPACE
 
 class QFontEngineFTRawFont;
 class QFontconfigDatabase;
+class QColrPaintGraphRenderer;
+
+#if defined(FT_COLOR_H) && (FREETYPE_MAJOR*10000 + FREETYPE_MINOR*100 + FREETYPE_PATCH) >= 21300
+#  define QFONTENGINE_FT_SUPPORT_COLRV1
+#endif
 
 /*
  * This class represents one font file on disk (like Arial.ttf) and is shared between all the font engines
@@ -62,6 +72,7 @@ public:
     }
 
     FT_Face face;
+    FT_MM_Var *mm_var;
     int xsize; // 26.6
     int ysize; // 26.6
     FT_Matrix matrix;
@@ -80,6 +91,11 @@ public:
     static void addGlyphToPath(FT_Face face, FT_GlyphSlot g, const QFixedPoint &point, QPainterPath *path, FT_Fixed x_scale, FT_Fixed y_scale);
     static void addBitmapToPath(FT_GlyphSlot slot, const QFixedPoint &point, QPainterPath *path);
 
+    inline QList<QFontVariableAxis> variableAxes() const
+    {
+        return variableAxisList;
+    }
+
 private:
     friend class QFontEngineFT;
     friend class QtFreetypeData;
@@ -91,6 +107,7 @@ private:
     QByteArray fontData;
 
     QFontEngine::Holder hbFace;
+    QList<QFontVariableAxis> variableAxisList;
 };
 
 class Q_GUI_EXPORT QFontEngineFT : public QFontEngine
@@ -172,6 +189,7 @@ private:
     QFixed underlinePosition() const override;
 
     glyph_t glyphIndex(uint ucs4) const override;
+    QString glyphName(glyph_t index) const override;
     void doKerning(QGlyphLayout *, ShaperFlags) const override;
 
     void getUnscaledGlyph(glyph_t glyph, QPainterPath *path, glyph_metrics_t *metrics) override;
@@ -183,7 +201,7 @@ private:
     void addOutlineToPath(qreal x, qreal y, const QGlyphLayout &glyphs,
                   QPainterPath *path, QTextItem::RenderFlags flags) override;
 
-    bool stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs, ShaperFlags flags) const override;
+    int stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs, int *nglyphs, ShaperFlags flags) const override;
 
     glyph_metrics_t boundingBox(const QGlyphLayout &glyphs) override;
     glyph_metrics_t boundingBox(glyph_t glyph) override;
@@ -211,6 +229,8 @@ private:
 
     int glyphCount() const override;
 
+    QList<QFontVariableAxis> variableAxes() const override;
+
     enum Scaling {
         Scaled,
         Unscaled
@@ -230,10 +250,19 @@ private:
                             GlyphFormat format = Format_None,
                             bool fetchMetricsOnly = false,
                             bool disableOutlineDrawing = false) const
-    { return loadGlyph(cacheEnabled ? &defaultGlyphSet : nullptr, glyph, subPixelPosition, format, fetchMetricsOnly, disableOutlineDrawing); }
+    {
+        return loadGlyph(cacheEnabled ? &defaultGlyphSet : nullptr,
+                         glyph,
+                         subPixelPosition,
+                         QColor(),
+                         format,
+                         fetchMetricsOnly,
+                         disableOutlineDrawing);
+    }
     Glyph *loadGlyph(QGlyphSet *set,
                      uint glyph,
                      const QFixedPoint &subPixelPosition,
+                     QColor color,
                      GlyphFormat = Format_None,
                      bool fetchMetricsOnly = false,
                      bool disableOutlineDrawing = false) const;
@@ -241,6 +270,7 @@ private:
                         const QFixedPoint &subPixelPosition,
                         GlyphFormat format,
                         const QTransform &t,
+                        QColor color,
                         bool fetchBoundingBox = false,
                         bool disableOutlineDrawing = false);
 
@@ -266,7 +296,7 @@ private:
     HintStyle defaultHintStyle() const { return default_hint_style; }
 
     static QFontEngineFT *create(const QFontDef &fontDef, FaceId faceId, const QByteArray &fontData = QByteArray());
-    static QFontEngineFT *create(const QByteArray &fontData, qreal pixelSize, QFont::HintingPreference hintingPreference);
+    static QFontEngineFT *create(const QByteArray &fontData, qreal pixelSize, QFont::HintingPreference hintingPreference, const QMap<QFont::Tag, float> &variableAxisValue);
 
 protected:
 
@@ -295,6 +325,23 @@ private:
     QFixed scaledBitmapMetrics(QFixed m) const;
     glyph_metrics_t scaledBitmapMetrics(const glyph_metrics_t &m, const QTransform &matrix) const;
 
+#if defined(QFONTENGINE_FT_SUPPORT_COLRV1)
+    Glyph *loadColrv1Glyph(QGlyphSet *set,
+                           Glyph *g,
+                           uint glyph,
+                           const QColor &color,
+                           bool fetchMetricsOnly) const;
+
+    bool traverseColr1(FT_OpaquePaint paint,
+                       QSet<QPair<FT_Byte *, FT_Bool> > *loops,
+                       QColor foregroundColor,
+                       FT_Color *palette,
+                       ushort paletteCount,
+                       QColrPaintGraphRenderer *paintGraphRenderer) const;
+    mutable glyph_t colrv1_bounds_cache_id = 0;
+    mutable QRect colrv1_bounds_cache;
+#endif // QFONTENGINE_FT_SUPPORT_COLRV1
+
     GlyphFormat defaultFormat;
     FT_Matrix matrix;
 
