@@ -1,3 +1,6 @@
+#ifndef QT_FEATURES_Gui_src_gui_qtgui_config_p_h_H
+#define QT_FEATURES_Gui_src_gui_qtgui_config_p_h_H
+
 #define QT_FEATURE_accessibility_atspi_bridge 1
 
 #define QT_FEATURE_directfb -1
@@ -6,12 +9,18 @@
 
 #define QT_FEATURE_directwrite3 -1
 
+#define QT_FEATURE_directwritecolrv1 -1
+
 #define QT_FEATURE_direct2d -1
 
 #define QT_FEATURE_direct2d1_1 -1
 
+#define QT_FEATURE_emojisegmenter 1
+
 #define QT_FEATURE_evdev -1
 
+#define QT_FEATURE_vxworksevdev -1
+
 #define QT_FEATURE_freetype 1
 
 #define QT_FEATURE_system_freetype 1
@@ -118,5 +127,41 @@
 
 #define QT_FEATURE_raster_fp 1
 
+#define QT_FEATURE_qtgui_threadpool 1
+
 #define QT_FEATURE_graphicsframecapture -1
 
+#define QT_FEATURE_wayland_client 1
+
+#define QT_FEATURE_wayland_server 1
+
+#define QT_FEATURE_wayland_egl 1
+
+#define QT_FEATURE_wayland_brcm -1
+
+#define QT_FEATURE_wayland_drm_egl_server_buffer 1
+
+#define QT_FEATURE_wayland_libhybris_egl_server_buffer -1
+
+#define QT_FEATURE_wayland_dmabuf_server_buffer 1
+
+#define QT_FEATURE_wayland_shm_emulation_server_buffer 1
+
+#define QT_FEATURE_wayland_vulkan_server_buffer 1
+
+#define QT_FEATURE_wayland_datadevice 1
+
+#define QT_FEATURE_wayland_client_primary_selection 1
+
+#define QT_FEATURE_wayland_client_fullscreen_shell_v1 1
+
+#define QT_FEATURE_wayland_client_wl_shell 1
+
+#define QT_FEATURE_wayland_client_xdg_shell 1
+
+#define QT_FEATURE_egl_extension_platform_wayland 1
+
+#define QT_FEATURE_run_opengl_tests 1
+
+
+#endif // QT_FEATURES_Gui_src_gui_qtgui_config_p_h_H
