@@ -1,3 +1,6 @@
+#ifndef QT_FEATURES_Network_src_network_qtnetwork_config_h_H
+#define QT_FEATURES_Network_src_network_qtnetwork_config_h_H
+
 #define QT_FEATURE_getifaddrs 1
 
 #define QT_FEATURE_ipv6ifname 1
@@ -41,3 +44,5 @@
 #define QT_NO_SCTP 1
 #define QT_NO_GSSAPI 1
 #define QT_NO_SSPI 1
+
+#endif // QT_FEATURES_Network_src_network_qtnetwork_config_h_H
