@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QSCROLLAREA_P_H
 #define QSCROLLAREA_P_H
@@ -20,6 +21,8 @@
 #include "private/qabstractscrollarea_p.h"
 #include <QtWidgets/qscrollbar.h>
 
+#include <QtCore/qpointer.h>
+
 QT_REQUIRE_CONFIG(scrollarea);
 
 QT_BEGIN_NAMESPACE
