@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #ifndef QWIDGETACTION_P_H
 #define QWIDGETACTION_P_H
@@ -18,6 +19,8 @@
 #include <QtWidgets/private/qtwidgetsglobal_p.h>
 #include "private/qaction_widgets_p.h"
 
+#include <QtCore/qpointer.h>
+
 QT_REQUIRE_CONFIG(action);
 
 QT_BEGIN_NAMESPACE
@@ -32,7 +35,7 @@ public:
     uint defaultWidgetInUse : 1;
     uint autoCreated : 1; // created by QToolBar::addWidget and the like
 
-    inline void _q_widgetDestroyed(QObject *o) {
+    inline void widgetDestroyed(QObject *o) {
         createdWidgets.removeAll(static_cast<QWidget *>(o));
     }
 };
